import 'node_modules/dhtmlx-scheduler-ultimate/codebase/ext/dhtmlxscheduler_all_timed';

import { DatePipe } from '@angular/common';
import {
  AfterViewInit, Component, ElementRef, OnChanges, OnDestroy, OnInit, SimpleChanges, ViewChild
} from '@angular/core';
import { MatDialog } from '@angular/material/dialog';
import { SchedulerStatic } from 'dhtmlx-scheduler-ultimate';
import { EventScheduler } from '../Model/EventScheduler';
import { SchedulerAppService } from '../schedulerApp.service';
import moment from 'moment';
import { DatUtils } from '../Model/DateUtils';
import { UpdateEventDialogComponent } from './update-event-dialog/update-event-dialog.component';

@Component({
  selector: 'app-scheduler',
  templateUrl: './scheduler.component.html',
  styleUrls: ['./scheduler.component.scss'],
  providers: [DatePipe]
})
export class SchedulerComponent implements OnInit, OnDestroy, OnChanges, AfterViewInit {
  @ViewChild('mainScheduler') schedulerContainer: ElementRef;
  events: EventScheduler[];

  schedulerTimeFilterOpenEvent: MouseEvent;
  schedulerDateFilterOpenEvent: MouseEvent;

  scheduler: SchedulerStatic;
  attachedEvents: string[] = [];

  date: Date;
  readonly heureDebut: string = '0700';
  readonly heureFin: string = '2059';

  typeAffichage = 'month';

  constructor(
    public datepipe: DatePipe,
    private _dialog: MatDialog,
    private _schedulerService: SchedulerAppService
  ) {
  }

  ngOnDestroy(): void {
    // On détache les event du scheduler
    this.attachedEvents.forEach((id: string) => {
      this.scheduler.detachEvent(id);
    });
  }

  ngOnInit(): void {
    this._initData();
  }

  ngOnChanges(changes: SimpleChanges): void {
    if (!changes.events.firstChange) {
      this._initScheduler();
      this._addEvents(this.events);

      // re-paramétrer le type d'affichage
      if (this.typeAffichage === 'month')
        this.affichageMois();
      else
        this.affichageSemaine();
    }
  }

  ngAfterViewInit(): void {
    this._initScheduler();;
    this._addEvents(this.events);
  }

  affichageSemaine() {
    const currentDate = this.date;
    this.scheduler.setCurrentView(currentDate, 'week');
    this.typeAffichage = 'week';
  }

  affichageMois() {
    const currentDate = this.date;
    this.scheduler.setCurrentView(currentDate, 'month');
    this.typeAffichage = 'month';
  }

  hideTooltip() {
    (this.scheduler as any).dhtmlXTooltip.hide();
  }

  private _initScheduler(): void {
    this.scheduler = this._schedulerService.SchedulerInstanceMain;

    this._configurerScheduler();

    this.scheduler.init(this.schedulerContainer.nativeElement, new Date(), this.typeAffichage);
  }

  private _configurerScheduler(): void {
    this.scheduler.config.edit_on_create = false;
    this.scheduler.config.mark_now = false;
    this.scheduler.config.dblclick_create = false;
    this.scheduler.locale.labels.new_event = null;
    this.scheduler.locale.labels.confirm_deleting = null;

    this.scheduler.config.key_nav = true;
    this.scheduler.config.drag_resize = true;

    // restreindre les modifications
    this.scheduler.config.readonly = true;
    this.scheduler.config.dblclick_create = false;
    this.scheduler.config.drag_create = false;
    this.scheduler.config.drag_move = false;
    this.scheduler.config.drag_resize = false;
    this.scheduler.xy.menu_width = 0;

    this.scheduler.config.multi_day = true;
    this.scheduler.config.all_timed = true;
    this.scheduler.config.full_day = true;

    this.scheduler.templates.week_date = (start, end) => moment(start).add(3, 'days').format('YYYY') + ' - Semaine '
      + DatUtils.getNumWeekFromDate(start);
    this.scheduler.templates.week_scale_date = (date) => this.scheduler.date.date_to_str('%D %j %F')(date);

    this._setTemplate();
    this._setTooltips();
  }

  private _addEvents(events: EventScheduler[]) {
    // create ranges for each period
    events?.forEach(event => {
      // set start and end dates and times
      const dateDebut: Date = DatUtils.getDateFromNumber(event.DateStart);
      const dateFin: Date = DatUtils.getDateFromNumber(event.DateEnd);
      if (event.HourStart)
        dateDebut.setHours(Number(event.HourStart.substring(0, 2)), Number(event.HourStart.substring(2, 4)));

      if (event.HourEnd)
        dateFin.setHours(Number(event.HourEnd.substring(0, 2)), Number(event.HourEnd.substring(2, 4)));

      this.CreateNewEventScheduler(dateDebut, dateFin, event);
    });
  }

  private CreateNewEventScheduler(dateDebut: Date, dateFin: Date, event: EventScheduler) {
    this.scheduler.addEvent({
      start_date: this.datepipe.transform(dateDebut, 'dd-MM-yyyy HH:mm'),
      end_date: this.datepipe.transform(dateFin, 'dd-MM-yyyy HH:mm'),
      text: event.Name,
      classType: event.Type,
      event
    });
  }

  private _setTooltips() {
    this.scheduler.attachEvent('onMouseMove', (id, e) => {

      const tooltip = (this.scheduler as any).dhtmlXTooltip;

      // if we are over event
      if (id) {
        const eventObj = this.scheduler.getEvent(id);
        const text = this._contentTooltips(eventObj);
        tooltip.delay(tooltip.show, tooltip, [e, text]);
      }
    });
  }
  private _contentTooltips(eventObj: any): string {
    let text = '<span class="bold ' + eventObj.classType + '-text">' + eventObj.event.Name + '</span><br/>';
    text += '<p>';

    const dateDebut = DatUtils.getDateFromNumber(eventObj.event.DateStart);
    const dateFin = DatUtils.getDateFromNumber(eventObj.event.DateEnd);

    text += 'Du <b>' + this.datepipe.transform(dateDebut, 'dd/MM/yyyy')
      + '</b> à ' + eventObj.event.HourStart.substring(0, 2) + 'H' + eventObj.event.HourStart.substring(2, 4)
      + '<br/>'
      + ' au <b>' + this.datepipe.transform(dateFin, 'dd/MM/yyyy')
      + '</b> à ' + eventObj.event.HourEnd.substring(0, 2) + 'H' + eventObj.event.HourEnd.substring(2, 4);
    text += '</p>';
    return text;
  }

  private _setTemplate() {
    // TEMPLATE VUE SEMAINE & MOIS
    this.scheduler.templates.event_class = (start, end, ev) => {
      let classes = 'libelle';
      classes += ' ' + ev.classType;
      return classes;
    };

    // TEMPLATE VUE SEMAINE
    this._setTemplateVueSemaine();

    // TEMPLATE VUE MOIS
    this._setTemplateVueMois();
  }

  private _setTemplateVueSemaine() {
    // en-tête / heure
    this.scheduler.templates.event_header = (start, end, ev) => {
      if (ev && ev.event) {
        let texte: string = '<div>';

        // si même jour que la date de début
        const datDebutPeriode = DatUtils.getNumberFromDate(ev.start_date);
        const premierJour = datDebutPeriode === ev.event.DateStart;

        // si même jour que la date de fin
        const datFinPeriode = DatUtils.getNumberFromDate(ev.end_date);
        const dernierJour = datFinPeriode === ev.event.DateEnd;

        if (premierJour && dernierJour && ev.event.HourStart && ev.event.HourEnd)
          texte += ev.event.HourStart.substring(0, 2) + 'H' + ev.event.HourStart.substring(2, 4) + ' - '
            + ev.event.HourEnd.substring(0, 2) + 'H' + ev.event.HourEnd.substring(2, 4);
        else if (premierJour && ev.event.HourStart)
          texte += 'à partir de ' + ev.event.HourStart.substring(0, 2) + 'H' + ev.event.HourStart.substring(2, 4);
        else if (dernierJour && ev.event.HourEnd)
          texte += 'jusqu\'à ' + ev.event.HourEnd.substring(0, 2) + 'H' + ev.event.HourEnd.substring(2, 4);
        texte += '</div>';
        return texte;
      }
      else
        return null;
    };
    // texte
    this.scheduler.templates.event_text = (start, end, ev) => ev.text
      ;
  }

  private _setTemplateVueMois() {
    // texte
    this.scheduler.templates.event_bar_text = (start, end, ev) => ev.text;
  }

  addRythm() {
    let dialogRef = this._dialog.open(UpdateEventDialogComponent, {
      width: '900px'
    });

    dialogRef.afterClosed().subscribe(result => {
      // simulation of the return of new events by the server
      const events = { ...this.events };
      this.events = events; // execution of the ngOnChanges
    });
  }

  private _initData() {
    const today = moment();
    let datTempMoment = today.add('month', -1);
    this.events = [];
    for (let i: number = 0; i < 20; i++) {
      let event = new EventScheduler();
      event.DateStart = DatUtils.getNumberFromDate(datTempMoment.toDate());
      datTempMoment.add('week', 1);
      event.DateEnd = DatUtils.getNumberFromDate(datTempMoment.toDate());
      event.HourStart = '1800';
      event.HourEnd = '1800';
      event.Name = i % 2 === 0 ? 'A' : 'B';
      event.Type = i % 2 === 0 ? 'typeA' : 'typeB';
      this.events.push(event);
    }
  }
}

